@@ -33,7 +33,7 @@ public class BannedPlugins
     extends BannedDependencies
 {
 
-    protected Set getDependenciesToCheck( MavenProject project )
+    protected Set<Artifact> getDependenciesToCheck( MavenProject project )
     {
         return project.getPluginArtifacts();
     }
