@@ -21,6 +21,7 @@ package org.apache.maven.plugins.enforcer;
 
 import java.io.File;
 import java.io.IOException;
+import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -53,6 +54,7 @@ import org.apache.maven.model.BuildBase;
 import org.apache.maven.model.Model;
 import org.apache.maven.model.Plugin;
 import org.apache.maven.model.Profile;
+import org.apache.maven.model.ReportPlugin;
 import org.apache.maven.plugin.InvalidPluginException;
 import org.apache.maven.plugin.MojoExecutionException;
 import org.apache.maven.plugin.PluginManager;
@@ -76,59 +78,90 @@ import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
  * This rule will enforce that all plugins specified in the poms have a version declared.
  *
  * @author <a href="mailto:brianf@apache.org">Brian Fox</a>
- * @version $Id: RequirePluginVersions.java 1028144 2010-10-28 00:10:58Z brianf $
+ * @version $Id: RequirePluginVersions.java 1697215 2015-08-23 16:27:17Z khmarbaise $
  */
 public class RequirePluginVersions
     extends AbstractNonCacheableEnforcerRule
 {
 
-    /** Don't allow the LATEST identifier. */
-    public boolean banLatest = true;
+    /**
+     * Don't allow the LATEST identifier.
+     * 
+     * @see {@link #setBanLatest(boolean)}
+     * @see {@link #isBanLatest()}
+     */
+    private boolean banLatest = true;
 
-    /** Don't allow the RELEASE identifier. */
-    public boolean banRelease = true;
+    /**
+     * Don't allow the RELEASE identifier.
+     * 
+     * @see {@link #setBanRelease(boolean)}
+     * @see {@link #isBanRelease()}
+     */
+    private boolean banRelease = true;
 
-    /** Don't allow snapshot plugins. */
-    public boolean banSnapshots = true;
+    /**
+     * Don't allow snapshot plugins.
+     * 
+     * @see {@link #setBanSnapshots(boolean)}
+     * @see {@link #isBanSnapshots()}
+     */
+    private boolean banSnapshots = true;
 
-    /** Don't allow timestamp snapshot plugins. */
-    public boolean banTimestamps = true;
+    /**
+     * Don't allow timestamp snapshot plugins.
+     * 
+     * @see {@link #setBanTimestamps(boolean)}
+     * @see {@link #isBanTimestamps()}
+     */
+    private boolean banTimestamps = true;
 
     /**
      * The comma separated list of phases that should be used to find lifecycle plugin bindings. The default value is
      * "clean,deploy,site".
+     * 
+     * @see {@link #setPhases(String)}
+     * @see {@link #getPhases()}
      */
-    public String phases = "clean,deploy,site";
+    private String phases = "clean,deploy,site";
 
     /**
      * Additional plugins to enforce have versions. These are plugins that may not be in the poms but are used anyway,
      * like help, eclipse etc. <br>
      * The plugins should be specified in the form: <code>group:artifactId</code>.
+     * 
+     * @see {@link #setAdditionalPlugins(List)}
+     * @see {@link #getAdditionalPlugins()}
      */
-    public List additionalPlugins;
+    private List<String> additionalPlugins;
 
     /**
      * Plugins to skip for version enforcement. The plugins should be specified in the form:
      * <code>group:artifactId</code>. NOTE: This is deprecated, use unCheckedPluginList instead.
-     * @deprecated
+     * 
+     * @see {@link #setUnCheckedPlugins(List)}
+     * @see {@link #getUnCheckedPlugins()}
      */
-    public List unCheckedPlugins;
+    private List unCheckedPlugins;
 
     /**
      * Same as unCheckedPlugins but as a comma list to better support properties. Sample form:
      * <code>group:artifactId,group2:artifactId2</code>
+     * 
      * @since 1.0-beta-1
+     * @see {@link #setUnCheckedPlugins(List)}
+     * @see {@link #getUnCheckedPlugins()}
      */
-    public String unCheckedPluginList;
+    private String unCheckedPluginList;
 
     /** The plugin manager. */
     private PluginManager pluginManager;
 
     /** The phase to lifecycle map. */
-    private Map phaseToLifecycleMap;
+    private Map<String, Lifecycle> phaseToLifecycleMap;
 
     /** The lifecycles. */
-    private List lifecycles;
+    private Collection<Lifecycle> lifecycles;
 
     /** The factory. */
     ArtifactFactory factory;
@@ -140,7 +173,7 @@ public class RequirePluginVersions
     ArtifactRepository local;
 
     /** The remote repositories. */
-    List remoteRepositories;
+    List<ArtifactRepository> remoteRepositories;
 
     /** The log. */
     Log log;
@@ -153,8 +186,8 @@ public class RequirePluginVersions
 
     /*
      * (non-Javadoc)
-     *
-     * @see org.apache.maven.enforcer.rule.api.EnforcerRule#execute(org.apache.maven.enforcer.rule.api.EnforcerRuleHelper)
+     * @see
+     * org.apache.maven.enforcer.rule.api.EnforcerRule#execute(org.apache.maven.enforcer.rule.api.EnforcerRuleHelper)
      */
     public void execute( EnforcerRuleHelper helper )
         throws EnforcerRuleException
@@ -169,21 +202,23 @@ public class RequirePluginVersions
             project = (MavenProject) helper.evaluate( "${project}" );
             LifecycleExecutor life;
             life = (LifecycleExecutor) helper.getComponent( LifecycleExecutor.class );
-            try
+
+            // The lifecycle API changed from Maven 2 to 3 so we have to do a hack to figure
+            // out which one we're using.
+            Field field = ReflectionUtils.getFieldByNameIncludingSuperclasses( "defaultLifeCycles", life.getClass() );
+            if ( field != null ) // Using Maven 3
             {
-              lifecycles = (List) ReflectionUtils.getValueIncludingSuperclasses( "lifecycles", life );
+                Object defaultLifeCycles = ReflectionUtils.getValueIncludingSuperclasses( "defaultLifeCycles", life );
+                Map lifecyclesMap =
+                    (Map) ReflectionUtils.getValueIncludingSuperclasses( "lifecycles", defaultLifeCycles );
+                lifecycles = lifecyclesMap.values();
             }
-            catch (Exception e)
+            else
+            // Using Maven 2
             {
-                log.info( "The requirePluginVersions rule is currently not compatible with Maven3.");
-                /*
-                 *
-                 * NOTE: If this happens, we're bailing out right away.
-                 *
-                 *
-                 */
-                return;
+                lifecycles = (Collection) ReflectionUtils.getValueIncludingSuperclasses( "lifecycles", life );
             }
+
             session = (MavenSession) helper.evaluate( "${session}" );
             pluginManager = (PluginManager) helper.getComponent( PluginManager.class );
             factory = (ArtifactFactory) helper.getComponent( ArtifactFactory.class );
@@ -194,15 +229,16 @@ public class RequirePluginVersions
             utils = new EnforcerRuleUtils( helper );
 
             // get all the plugins that are bound to the specified lifecycles
-            Set allPlugins = getBoundPlugins( life, project, phases );
+            Set<Plugin> allPlugins = getBoundPlugins( life, project, phases );
 
             // insert any additional plugins specified by the user.
             allPlugins = addAdditionalPlugins( allPlugins, additionalPlugins );
             allPlugins.addAll( getProfilePlugins( project ) );
 
-
             // pull out any we should skip
-            allPlugins = (Set) removeUncheckedPlugins( combineUncheckedPlugins( unCheckedPlugins, unCheckedPluginList ), allPlugins );
+            allPlugins =
+                (Set) removeUncheckedPlugins( combineUncheckedPlugins( unCheckedPlugins, unCheckedPluginList ),
+                                              allPlugins );
 
             // there's nothing to do here
             if ( allPlugins.isEmpty() )
@@ -216,15 +252,12 @@ public class RequirePluginVersions
             }
 
             // get all the plugins that are mentioned in the pom (and parents)
-            List pluginWrappers = getAllPluginEntries( project );
+            List<PluginWrapper> pluginWrappers = getAllPluginEntries( project );
 
             // now look for the versions that aren't valid and add to a list.
-            ArrayList failures = new ArrayList();
-            Iterator iter = allPlugins.iterator();
-            while ( iter.hasNext() )
+            List<Plugin> failures = new ArrayList<Plugin>();
+            for ( Plugin plugin : allPlugins )
             {
-                Plugin plugin = (Plugin) iter.next();
-
                 if ( !hasValidVersionSpecified( helper, plugin, pluginWrappers ) )
                 {
                     failures.add( plugin );
@@ -234,7 +267,7 @@ public class RequirePluginVersions
             // if anything was found, log it then append the optional message.
             if ( !failures.isEmpty() )
             {
-                StringBuffer newMsg = new StringBuffer();
+                StringBuilder newMsg = new StringBuilder();
                 newMsg.append( "Some plugins are missing valid versions:" );
                 if ( banLatest || banRelease || banSnapshots || banTimestamps )
                 {
@@ -253,11 +286,8 @@ public class RequirePluginVersions
                     }
                     newMsg.append( "are not allowed )\n" );
                 }
-                iter = failures.iterator();
-                while ( iter.hasNext() )
+                for ( Plugin plugin : failures )
                 {
-                    Plugin plugin = (Plugin) iter.next();
-
                     newMsg.append( plugin.getGroupId() );
                     newMsg.append( ":" );
                     newMsg.append( plugin.getArtifactId() );
@@ -286,6 +316,7 @@ public class RequirePluginVersions
                     }
                     newMsg.append( "\n" );
                 }
+                String message = getMessage();
                 if ( StringUtils.isNotEmpty( message ) )
                 {
                     newMsg.append( message );
@@ -341,18 +372,17 @@ public class RequirePluginVersions
      *
      * @param uncheckedPlugins
      * @param plugins
-     * @return
      * @throws MojoExecutionException
+     * @return The plugins which have been removed.
      */
-    public Collection removeUncheckedPlugins( Collection uncheckedPlugins, Collection plugins )
+    public Collection<Plugin> removeUncheckedPlugins( Collection<String> uncheckedPlugins, Collection<Plugin> plugins )
         throws MojoExecutionException
     {
         if ( uncheckedPlugins != null && !uncheckedPlugins.isEmpty() )
         {
-            Iterator iter = uncheckedPlugins.iterator();
-            while ( iter.hasNext() )
+            for ( String pluginKey : uncheckedPlugins )
             {
-                Plugin plugin = parsePluginString( (String) iter.next(), "UncheckedPlugins" );
+                Plugin plugin = parsePluginString( pluginKey, "UncheckedPlugins" );
                 plugins.remove( plugin );
             }
         }
@@ -361,21 +391,24 @@ public class RequirePluginVersions
 
     /**
      * Combines the old Collection with the new comma separated list.
+     * 
      * @param uncheckedPlugins
      * @param uncheckedPluginsList
-     * @return
+     * @return List of unchecked plugins.
      */
-    public Collection combineUncheckedPlugins( Collection uncheckedPlugins, String uncheckedPluginsList )
+    // CHECKSTYLE_OFF: LineLength
+    public Collection<String> combineUncheckedPlugins( Collection<String> uncheckedPlugins, String uncheckedPluginsList )
+    // CHECKSTYLE_ON: LineLength
     {
-        //if the comma list is empty, then there's nothing to do here.
+        // if the comma list is empty, then there's nothing to do here.
         if ( StringUtils.isNotEmpty( uncheckedPluginsList ) )
         {
-            //make sure there is a collection to add to.
+            // make sure there is a collection to add to.
             if ( uncheckedPlugins == null )
             {
-                uncheckedPlugins = new HashSet();
+                uncheckedPlugins = new HashSet<String>();
             }
-            else if (!uncheckedPlugins.isEmpty() && log != null)
+            else if ( !uncheckedPlugins.isEmpty() && log != null )
             {
                 log.warn( "The parameter 'unCheckedPlugins' is deprecated. Use 'unCheckedPluginList' instead" );
             }
@@ -393,20 +426,18 @@ public class RequirePluginVersions
      * @return the sets the
      * @throws MojoExecutionException the mojo execution exception
      */
-    public Set addAdditionalPlugins( Set existing, List additional )
+    public Set<Plugin> addAdditionalPlugins( Set<Plugin> existing, List<String> additional )
         throws MojoExecutionException
     {
         if ( additional != null )
         {
-            Iterator iter = additional.iterator();
-            while ( iter.hasNext() )
+            for ( String pluginString : additional )
             {
-                String pluginString = (String) iter.next();
                 Plugin plugin = parsePluginString( pluginString, "AdditionalPlugins" );
 
                 if ( existing == null )
                 {
-                    existing = new HashSet();
+                    existing = new HashSet<Plugin>();
                     existing.add( plugin );
                 }
                 else if ( !existing.contains( plugin ) )
@@ -422,8 +453,9 @@ public class RequirePluginVersions
      * Helper method to parse and inject a Plugin.
      *
      * @param pluginString
-     * @return
+     * @param field
      * @throws MojoExecutionException
+     * @return the plugin
      */
     protected Plugin parsePluginString( String pluginString, String field )
         throws MojoExecutionException
@@ -457,20 +489,19 @@ public class RequirePluginVersions
      * @param project the project
      * @return the profile plugins
      */
-    public Set getProfilePlugins( MavenProject project )
+    public Set<Plugin> getProfilePlugins( MavenProject project )
     {
-        Set result = new HashSet();
-        List profiles = project.getActiveProfiles();
+        Set<Plugin> result = new HashSet<Plugin>();
+        @SuppressWarnings( "unchecked" )
+        List<Profile> profiles = project.getActiveProfiles();
         if ( profiles != null && !profiles.isEmpty() )
         {
-            Iterator iter = profiles.iterator();
-            while ( iter.hasNext() )
+            for ( Profile p : profiles )
             {
-                Profile p = (Profile) iter.next();
                 BuildBase b = p.getBuild();
                 if ( b != null )
                 {
-                    List plugins = b.getPlugins();
+                    List<Plugin> plugins = b.getPlugins();
                     if ( plugins != null )
                     {
                         result.addAll( plugins );
@@ -494,8 +525,9 @@ public class RequirePluginVersions
         try
         {
             Model model = project.getModel();
-            Map plugins = model.getBuild().getPluginsAsMap();
-            found = (Plugin) plugins.get( plugin.getKey() );
+            @SuppressWarnings( "unchecked" )
+            Map<String, Plugin> plugins = model.getBuild().getPluginsAsMap();
+            found = plugins.get( plugin.getKey() );
         }
         catch ( NullPointerException e )
         {
@@ -520,7 +552,8 @@ public class RequirePluginVersions
     protected Plugin resolvePlugin( Plugin plugin, MavenProject project )
     {
 
-        List pluginRepositories = project.getPluginArtifactRepositories();
+        @SuppressWarnings( "unchecked" )
+        List<ArtifactRepository> pluginRepositories = project.getPluginArtifactRepositories();
         Artifact artifact =
             factory.createPluginArtifact( plugin.getGroupId(), plugin.getArtifactId(),
                                           VersionRange.createFromVersion( "LATEST" ) );
@@ -532,9 +565,11 @@ public class RequirePluginVersions
         }
         catch ( ArtifactResolutionException e )
         {
+            //What does this mean?
         }
         catch ( ArtifactNotFoundException e )
         {
+            //What does this mean?
         }
 
         return plugin;
@@ -552,11 +587,11 @@ public class RequirePluginVersions
      * @throws LifecycleExecutionException the lifecycle execution exception
      * @throws IllegalAccessException the illegal access exception
      */
-    protected Set getBoundPlugins( LifecycleExecutor life, MavenProject project, String thePhases )
+    protected Set<Plugin> getBoundPlugins( LifecycleExecutor life, MavenProject project, String thePhases )
         throws PluginNotFoundException, LifecycleExecutionException, IllegalAccessException
     {
 
-        Set allPlugins = new HashSet();
+        Set<Plugin> allPlugins = new HashSet<Plugin>();
 
         // lookup the bindings for all the passed in phases
         String[] lifecyclePhases = thePhases.split( "," );
@@ -593,15 +628,14 @@ public class RequirePluginVersions
      * @param pluginWrappers the plugins
      * @return true, if successful
      */
-    protected boolean hasValidVersionSpecified( EnforcerRuleHelper helper, Plugin source, List pluginWrappers )
+    protected boolean hasValidVersionSpecified( EnforcerRuleHelper helper, Plugin source,
+                                                List<PluginWrapper> pluginWrappers )
     {
         boolean found = false;
         boolean status = false;
-        Iterator iter = pluginWrappers.iterator();
-        while ( iter.hasNext() )
+        for ( PluginWrapper plugin : pluginWrappers )
         {
             // find the matching plugin entry
-            PluginWrapper plugin = (PluginWrapper) iter.next();
             if ( source.getArtifactId().equals( plugin.getArtifactId() )
                 && source.getGroupId().equals( plugin.getGroupId() ) )
             {
@@ -686,11 +720,14 @@ public class RequirePluginVersions
      * @throws PluginNotFoundException the plugin not found exception
      * @throws LifecycleExecutionException the lifecycle execution exception
      */
-    private Set getAllPlugins( MavenProject project, Lifecycle lifecycle )
+    @SuppressWarnings( "unchecked" )
+    private Set<Plugin> getAllPlugins( MavenProject project, Lifecycle lifecycle )
         throws PluginNotFoundException, LifecycleExecutionException
 
     {
-        HashSet plugins = new HashSet();
+        log.debug( "RequirePluginVersions.getAllPlugins:" );
+
+        Set<Plugin> plugins = new HashSet<Plugin>();
         // first, bind those associated with the packaging
         Map mappings = findMappingsForLifecycle( project, lifecycle );
 
@@ -698,7 +735,30 @@ public class RequirePluginVersions
         while ( iter.hasNext() )
         {
             Entry entry = (Entry) iter.next();
-            String value = (String) entry.getValue();
+            log.debug( "  lifecycleMapping = " + entry.getKey() );
+            String pluginsForLifecycle = (String) entry.getValue();
+            log.debug( "  plugins = " + pluginsForLifecycle );
+            if ( StringUtils.isNotEmpty( pluginsForLifecycle ) )
+            {
+                String pluginList[] = pluginsForLifecycle.split( "," );
+                for ( String plugin : pluginList )
+                {
+                    plugin = StringUtils.strip( plugin );
+                    log.debug( "    plugin = " + plugin );
+                    String tokens[] = plugin.split( ":" );
+                    log.debug( "    GAV = " + Arrays.asList( tokens ) );
+
+                    Plugin p = new Plugin();
+                    p.setGroupId( tokens[0] );
+                    p.setArtifactId( tokens[1] );
+                    plugins.add( p );
+                }
+            }
+        }
+
+        List<String> mojos = findOptionalMojosForLifecycle( project, lifecycle );
+        for ( String value : mojos )
+        {
             String tokens[] = value.split( ":" );
 
             Plugin plugin = new Plugin();
@@ -707,23 +767,7 @@ public class RequirePluginVersions
             plugins.add( plugin );
         }
 
-        List mojos = findOptionalMojosForLifecycle( project, lifecycle );
-        iter = mojos.iterator();
-        while ( iter.hasNext() )
-        {
-            String value = (String) iter.next();
-            String tokens[] = value.split( ":" );
-
-            Plugin plugin = new Plugin();
-            plugin.setGroupId( tokens[0] );
-            plugin.setArtifactId( tokens[1] );
-            plugins.add( plugin );
-        }
-
-        for ( Iterator i = project.getBuildPlugins().iterator(); i.hasNext(); )
-        {
-            plugins.add( i.next() );
-        }
+        plugins.addAll( project.getBuildPlugins() );
 
         return plugins;
     }
@@ -738,21 +782,19 @@ public class RequirePluginVersions
      * @return the phase to lifecycle map
      * @throws LifecycleExecutionException the lifecycle execution exception
      */
-    public Map getPhaseToLifecycleMap()
+    public Map<String, Lifecycle> getPhaseToLifecycleMap()
         throws LifecycleExecutionException
     {
         if ( phaseToLifecycleMap == null )
         {
-            phaseToLifecycleMap = new HashMap();
+            phaseToLifecycleMap = new HashMap<String, Lifecycle>();
 
-            for ( Iterator i = lifecycles.iterator(); i.hasNext(); )
+            for ( Lifecycle lifecycle : lifecycles )
             {
-                Lifecycle lifecycle = (Lifecycle) i.next();
-
-                for ( Iterator p = lifecycle.getPhases().iterator(); p.hasNext(); )
+                @SuppressWarnings( "unchecked" )
+                List<String> phases = lifecycle.getPhases();
+                for ( String phase : phases )
                 {
-                    String phase = (String) p.next();
-
                     if ( phaseToLifecycleMap.containsKey( phase ) )
                     {
                         Lifecycle prevLifecycle = (Lifecycle) phaseToLifecycleMap.get( phase );
@@ -857,11 +899,12 @@ public class RequirePluginVersions
      * @throws LifecycleExecutionException the lifecycle execution exception
      * @throws PluginNotFoundException the plugin not found exception
      */
-    private List findOptionalMojosForLifecycle( MavenProject project, Lifecycle lifecycle )
+    @SuppressWarnings( "unchecked" )
+    private List<String> findOptionalMojosForLifecycle( MavenProject project, Lifecycle lifecycle )
         throws LifecycleExecutionException, PluginNotFoundException
     {
         String packaging = project.getPackaging();
-        List optionalMojos = null;
+        List<String> optionalMojos = null;
 
         LifecycleMapping m =
             (LifecycleMapping) findExtension( project, LifecycleMapping.ROLE, packaging, session.getSettings(),
@@ -882,13 +925,13 @@ public class RequirePluginVersions
             catch ( ComponentLookupException e )
             {
                 log.debug( "Error looking up lifecycle mapping to retrieve optional mojos. Lifecycle ID: "
-                    + lifecycle.getId() + ". Error: " + e.getMessage(), e );
+                               + lifecycle.getId() + ". Error: " + e.getMessage(), e );
             }
         }
 
         if ( optionalMojos == null )
         {
-            optionalMojos = Collections.EMPTY_LIST;
+            optionalMojos = Collections.emptyList();
         }
 
         return optionalMojos;
@@ -912,10 +955,10 @@ public class RequirePluginVersions
     {
         Object pluginComponent = null;
 
-        for ( Iterator i = project.getBuildPlugins().iterator(); i.hasNext() && pluginComponent == null; )
+        @SuppressWarnings( "unchecked" )
+        List<Plugin> buildPlugins = project.getBuildPlugins();
+        for ( Plugin plugin : buildPlugins )
         {
-            Plugin plugin = (Plugin) i.next();
-
             if ( plugin.isExtensions() )
             {
                 verifyPlugin( plugin, project, settings, localRepository );
@@ -927,6 +970,11 @@ public class RequirePluginVersions
                 try
                 {
                     pluginComponent = pluginManager.getPluginComponent( plugin, role, roleHint );
+
+                    if ( pluginComponent != null )
+                    {
+                        break;
+                    }
                 }
                 catch ( ComponentLookupException e )
                 {
@@ -1005,34 +1053,34 @@ public class RequirePluginVersions
      * @throws IOException Signals that an I/O exception has occurred.
      * @throws XmlPullParserException the xml pull parser exception
      */
-    protected List getAllPluginEntries( MavenProject project )
+    protected List<PluginWrapper> getAllPluginEntries( MavenProject project )
         throws ArtifactResolutionException, ArtifactNotFoundException, IOException, XmlPullParserException
     {
-        List plugins = new ArrayList();
+        List<PluginWrapper> plugins = new ArrayList<PluginWrapper>();
         // get all the pom models
-        
+
         String pomName = null;
         try
         {
             pomName = project.getFile().getName();
         }
-        catch (Exception e)
+        catch ( Exception e )
         {
             pomName = "pom.xml";
         }
-        List models =
+        List<Model> models =
             utils.getModelsRecursively( project.getGroupId(), project.getArtifactId(), project.getVersion(),
                                         new File( project.getBasedir(), pomName ) );
 
         // now find all the plugin entries, either in
         // build.plugins or build.pluginManagement.plugins, profiles.plugins and reporting
-        Iterator iter = models.iterator();
-        while ( iter.hasNext() )
+        for ( Model model : models )
         {
-            Model model = (Model) iter.next();
             try
             {
-                plugins.addAll( PluginWrapper.addAll( model.getBuild().getPlugins(), model.getId() + ".build.plugins" ) );
+                List<Plugin> modelPlugins = model.getBuild().getPlugins();
+                plugins.addAll( PluginWrapper.addAll( utils.resolvePlugins( modelPlugins ), model.getId()
+                    + ".build.plugins" ) );
             }
             catch ( NullPointerException e )
             {
@@ -1041,8 +1089,10 @@ public class RequirePluginVersions
 
             try
             {
+                List<ReportPlugin> modelReportPlugins = model.getReporting().getPlugins();
                 // add the reporting plugins
-                plugins.addAll( PluginWrapper.addAll( model.getReporting().getPlugins(), model.getId() + ".reporting" ) );
+                plugins.addAll( PluginWrapper.addAll( utils.resolveReportPlugins( modelReportPlugins ), model.getId()
+                    + ".reporting" ) );
             }
             catch ( NullPointerException e )
             {
@@ -1051,8 +1101,9 @@ public class RequirePluginVersions
 
             try
             {
-                plugins.addAll( PluginWrapper.addAll( model.getBuild().getPluginManagement().getPlugins(),
-                                                      model.getId() + ".build.pluginManagement.plugins" ) );
+                List<Plugin> modelPlugins = model.getBuild().getPluginManagement().getPlugins();
+                plugins.addAll( PluginWrapper.addAll( utils.resolvePlugins( modelPlugins ), model.getId()
+                    + ".build.pluginManagement.plugins" ) );
             }
             catch ( NullPointerException e )
             {
@@ -1060,13 +1111,13 @@ public class RequirePluginVersions
             }
 
             // Add plugins in profiles
-            Iterator it = model.getProfiles().iterator();
-            while ( it.hasNext() )
+            List<Profile> profiles = model.getProfiles();
+            for ( Profile profile : profiles )
             {
-                Profile profile = (Profile) it.next();
                 try
                 {
-                    plugins.addAll( PluginWrapper.addAll( profile.getBuild().getPlugins(), model.getId()
+                    List<Plugin> modelPlugins = profile.getBuild().getPlugins();
+                    plugins.addAll( PluginWrapper.addAll( utils.resolvePlugins( modelPlugins ), model.getId()
                         + ".profiles.profile[" + profile.getId() + "].build.plugins" ) );
                 }
                 catch ( NullPointerException e )
@@ -1076,9 +1127,11 @@ public class RequirePluginVersions
 
                 try
                 {
+                    List<ReportPlugin> modelReportPlugins = profile.getReporting().getPlugins();
                     // add the reporting plugins
-                    plugins.addAll( PluginWrapper.addAll( profile.getReporting().getPlugins(), model.getId()
-                        + "profile[" + profile.getId() + "].reporting.plugins" ) );
+                    plugins.addAll( PluginWrapper.addAll( utils.resolveReportPlugins( modelReportPlugins ),
+                                                          model.getId() + "profile[" + profile.getId()
+                                                              + "].reporting.plugins" ) );
                 }
                 catch ( NullPointerException e )
                 {
@@ -1086,10 +1139,9 @@ public class RequirePluginVersions
                 }
                 try
                 {
-                    // add the reporting plugins
-                    plugins.addAll( PluginWrapper.addAll( profile.getBuild().getPluginManagement().getPlugins(),
-                                                          model.getId() + "profile[" + profile.getId()
-                                                              + "].build.pluginManagement.plugins" ) );
+                    List<Plugin> modelPlugins = profile.getBuild().getPluginManagement().getPlugins();
+                    plugins.addAll( PluginWrapper.addAll( utils.resolvePlugins( modelPlugins ), model.getId()
+                        + "profile[" + profile.getId() + "].build.pluginManagement.plugins" ) );
                 }
                 catch ( NullPointerException e )
                 {
@@ -1141,26 +1193,6 @@ public class RequirePluginVersions
         this.banRelease = theBanRelease;
     }
 
-    /**
-     * Gets the message.
-     *
-     * @return the message
-     */
-    protected String getMessage()
-    {
-        return this.message;
-    }
-
-    /**
-     * Sets the message.
-     *
-     * @param theMessage the message to set
-     */
-    protected void setMessage( String theMessage )
-    {
-        this.message = theMessage;
-    }
-
     /**
      * Gets the utils.
      *
@@ -1230,4 +1262,24 @@ public class RequirePluginVersions
     {
         this.unCheckedPlugins = unCheckedPlugins;
     }
+
+    public final void setPhases( String phases )
+    {
+        this.phases = phases;
+    }
+
+    public final String getPhases()
+    {
+        return phases;
+    }
+
+    public final void setAdditionalPlugins( List<String> additionalPlugins )
+    {
+        this.additionalPlugins = additionalPlugins;
+    }
+
+    public final List<String> getAdditionalPlugins()
+    {
+        return additionalPlugins;
+    }
 }
\ No newline at end of file
